#
#  Copyright (c) 2018-2024 Renesas Inc.
#  Copyright (c) 2018-2024 EPAM Systems Inc.
#
import tempfile
import unittest

from aos_prov.utils.errors import UserCredentialsError
from aos_prov.utils.user_credentials import TempCredentials, UserCredentials
from cryptography.hazmat.primitives._serialization import (
    Encoding,
    NoEncryption,
    PrivateFormat,
)
from tests.fixtures import (
    generate_ca_key_and_cert,
    generate_test_key_certificate,
    key_cert_to_pem,
    key_cert_to_pkcs12,
)


class TestTempCredentials(unittest.TestCase):
    def test_context_management(self):
        ca_key, ca_cert = generate_ca_key_and_cert()
        key, cert = generate_test_key_certificate(ca_key)

        cert_pem_bytes = cert.public_bytes(encoding=Encoding.PEM)

        private_key_pem_bytes = key.private_bytes(
            encoding=Encoding.PEM,
            format=PrivateFormat.PKCS8,
            encryption_algorithm=NoEncryption(),
        )
        with TempCredentials(cert_pem_bytes, private_key_pem_bytes, None, None) as temp_creds:
            self.assertIsNotNone(temp_creds.cert_file_name)
            self.assertIsNotNone(temp_creds.key_file_name)


class TestUserCredentials(unittest.TestCase):
    def test_create_with_pkcs12(self):
        ca_key, ca_cert = generate_ca_key_and_cert()
        key, cert = generate_test_key_certificate(ca_key)
        pkcs_12_cert = key_cert_to_pkcs12(key, cert, ca_cert)

        with tempfile.NamedTemporaryFile(delete=True) as temp_cert:
            temp_cert.write(pkcs_12_cert)
            temp_cert.seek(0)
            uc = UserCredentials('cert', 'key', temp_cert.name)

        uc.cert_type = 'type1'
        self.assertEqual(uc.cert_type, 'type1')

        uc.certificate = 'some cert 1'
        self.assertEqual(uc.certificate, 'some cert 1')

        uc.csr = 'csr'
        self.assertEqual(uc.csr, 'csr')

        self.assertEqual(uc.cloud_url, 'developer.test.local')
        self.assertIsNotNone(uc.user_credentials)

    def test_create_without_pkcs12(self):
        ca_key, ca_cert = generate_ca_key_and_cert()
        key, cert = generate_test_key_certificate(ca_key)

        cert_pem_bytes = cert.public_bytes(encoding=Encoding.PEM)
        cert_file = tempfile.NamedTemporaryFile(delete=True)
        cert_file.write(cert_pem_bytes)
        cert_file.seek(0)

        private_key_pem_bytes = key.private_bytes(
            encoding=Encoding.PEM,
            format=PrivateFormat.PKCS8,
            encryption_algorithm=NoEncryption(),
        )
        key_file = tempfile.NamedTemporaryFile(delete=True)
        key_file.write(private_key_pem_bytes)
        key_file.seek(0)

        uc1 = UserCredentials(cert_file.name, key_file.name, 'fake_pkcs12')

        self.assertEqual(uc1.cloud_url, 'developer.test.local')
        self.assertIsNotNone(uc1.user_credentials)

        uc2 = UserCredentials(cert_file.name, key_file.name, None)

        self.assertEqual(uc2.cloud_url, 'developer.test.local')
        self.assertIsNotNone(uc2.user_credentials)

        with self.assertRaises(UserCredentialsError) as exc:
            UserCredentials('cert_file', key_file.name, None)

        self.assertEqual(str(exc.exception), 'User credentials file cert_file not found.')

        with self.assertRaises(UserCredentialsError) as exc:
            UserCredentials(cert_file.name, 'key_file', None)

        self.assertEqual(str(exc.exception), 'User credentials file key_file not found.')

    def test_file_not_found(self):
        with self.assertRaises(UserCredentialsError) as exc:
            UserCredentials('cert', 'key', 'fake_pkcs12')

        self.assertEqual(str(exc.exception), 'User credentials file fake_pkcs12 not found.')

    def test_format_issues(self):
        ca_key, ca_cert = generate_ca_key_and_cert()
        key, cert = generate_test_key_certificate(ca_key)
        _, other_cert = generate_test_key_certificate(ca_key)

        cert_pem_bytes = cert.public_bytes(encoding=Encoding.PEM)
        cert_file = tempfile.NamedTemporaryFile(delete=True)
        cert_file.write(cert_pem_bytes)
        cert_file.seek(0)
        cert_file_corrupted = tempfile.NamedTemporaryFile(delete=True)
        cert_file_corrupted.write(b'INCORRECT_DATA')
        cert_file_corrupted.seek(0)

        other_cert_pem_bytes = other_cert.public_bytes(encoding=Encoding.PEM)
        other_cert_file = tempfile.NamedTemporaryFile(delete=True)
        other_cert_file.write(other_cert_pem_bytes)
        other_cert_file.seek(0)

        private_key_pem_bytes = key.private_bytes(
            encoding=Encoding.PEM,
            format=PrivateFormat.PKCS8,
            encryption_algorithm=NoEncryption(),
        )
        key_file = tempfile.NamedTemporaryFile(delete=True)
        key_file.write(private_key_pem_bytes)
        key_file.seek(0)
        key_file_corrupted = tempfile.NamedTemporaryFile(delete=True)
        key_file_corrupted.write(b'INCORRECT_DATA')
        key_file_corrupted.seek(0)

        with self.assertRaises(UserCredentialsError) as exc:
            UserCredentials(cert_file.name, key_file_corrupted.name, None)

        self.assertEqual(str(exc.exception), f'Private key is not correct: {key_file_corrupted.read().decode()}')

        with self.assertRaises(UserCredentialsError) as exc:
            UserCredentials(cert_file_corrupted.name, key_file.name, None)

        self.assertEqual(str(exc.exception), f'Certificate is not correct: {cert_file_corrupted.read()}')

        with self.assertRaises(UserCredentialsError) as exc:
            UserCredentials(other_cert_file.name, key_file.name, None)

        self.assertEqual(str(exc.exception), 'User private key does not match certificate')
